@@ -14,10 +14,8 @@
 
 import sys
 import os.path
-import re
 import xml.sax.saxutils
 import codecs
-import locale
 import gzip
 import zipfile
 import StringIO
@@ -41,6 +39,16 @@ for c in range(9) + range(11, 13) + range(14, 32):
 unEscDict = {'&quot;': '"'}
 encryptPrefix = '>>TL+enc'
 
+tabbedImport = 'readTabbed'
+tableImport = 'readTable'
+textLineImport = 'readLines'
+textParaImport = 'readPara'
+treepadImport = 'readTreepad'
+xbelImport = 'readXbel'
+mozillaImport = 'readMozilla'
+xmlImport = 'readXml'
+odfImport = 'readOdf'
+
 class TreeDoc(object):
     """Tree document class - stores root and has tree utilities"""
     passwordDict = {}
@@ -51,7 +59,10 @@ class TreeDoc(object):
     separatorName = _('SEPARATOR', 'bookmark format separator name')
     bookmarkRootTitle = _('Bookmarks')
     copyFormat = None
-    def __init__(self, filePath=None):
+    def __init__(self, filePath=None, setNewDefaults=False, importType=None):
+        """Open filePath (can also be file ref) if given,
+           setNewDefaults uses user defaults for compression & encryption,
+           importType gives an import method to read the file"""
         globalref.docRef = self
         self.root = None
         self.treeFormats = TreeFormats()
@@ -59,29 +70,33 @@ class TreeDoc(object):
         self.fileInfoFormat = None
         TreeDoc.copyFormat = nodeformat.NodeFormat('_DUMMY__ROOT_', {},
                                                    TreeFormats.fieldDefault)
-        # self.resetParam()
-        self.fileName = ''
-        if filePath:
-            self.readFile(filePath)
-        else:
-            self.newFile()
-
-    def resetParam(self):
-        """Set doc setting values to defaults"""
-        self.selection = treeselection.TreeSelection([self.root])
         self.undoStore = undo.UndoRedoStore()
         self.redoStore =  undo.UndoRedoStore()
-        self.modified = False
+        self.sortFields = ['']
+        self.fileName = ''
         self.spaceBetween = True
         self.lineBreaks = True
         self.formHtml = True
         self.childFieldSep = TreeDoc.childFieldSepDflt
         self.spellChkLang = ''
-        self.sortFields = ['']
-        self.xslCssLink = ''
         self.xlstLink = ''
+        self.xslCssLink = ''
         self.tlVersion = __version__
         self.fileInfoFormat = nodeformat.FileInfoFormat()
+        if filePath:
+            if importType:
+                getattr(self, importType)(filePath)
+            else:
+                self.readFile(filePath)
+        else:
+            self.treeFormats = TreeFormats({}, True)
+            self.root = TreeItem(None, TreeFormats.rootFormatDefault)
+            self.root.setTitle(TreeDoc.rootTitleDefault)
+        self.modified = False
+        if setNewDefaults or not hasattr(self, 'compressFile'):
+            self.compressFile = globalref.options.boolData('CompressNewFiles')
+            self.encryptFile = globalref.options.boolData('EncryptNewFiles')
+        self.selection = treeselection.TreeSelection([self.root])
         self.fileInfoFormat.translateFields()
         self.fileInfoFormat.updateFileInfo()
 
@@ -135,14 +150,12 @@ class TreeDoc(object):
         if self.encryptFile:
             password = TreeDoc.passwordDict.get(fileObj.name, '')
             if not password:
-                self.storedFileRef = fileObj
-                self.storedFileRef.seek(0)
+                fileObj.close()
                 raise PasswordError, 'Missing password'
             try:
                 text = p3.p3_decrypt(fileObj.read(), password)
             except p3.CryptError:
-                self.storedFileRef = fileObj
-                self.storedFileRef.seek(0)
+                fileObj.close()
                 raise PasswordError, 'Incorrect password'
             fileObj.close()
             fileObj = StringIO.StringIO(text)
@@ -167,28 +180,16 @@ class TreeDoc(object):
             fileRef.name = name
         return fileRef
 
-    def newFile(self):
-        """Start empty file"""
-        self.compressFile = globalref.options.boolData('CompressNewFiles')
-        self.encryptFile = globalref.options.boolData('EncryptNewFiles')
-        self.treeFormats = TreeFormats({}, True)
-        self.root = TreeItem(None, TreeFormats.rootFormatDefault)
-        self.root.setTitle(TreeDoc.rootTitleDefault)
-        self.resetParam()
-        self.fileName = ''
-
     def readFile(self, fileRef):
         """Open and read file - raise exception on failure,
            fileRef is either file path or file object"""
-        origFileInfoFormat = self.fileInfoFormat
-        self.fileInfoFormat = nodeformat.FileInfoFormat()
         filePath = hasattr(fileRef, 'read') and \
                    unicode(fileRef.name, sys.getfilesystemencoding()) or \
                    fileRef
         try:
             f = self.getReadFileObj(fileRef)
             f = self.decryptFile(f)
-            handler = treexmlparse.TreeSaxHandler()
+            handler = treexmlparse.TreeSaxHandler(self)
             input = xml.sax.InputSource()
             input.setByteStream(f)
             input.setEncoding('utf-8')
@@ -199,39 +200,20 @@ class TreeDoc(object):
         except IOError:
             print 'Error - could not read file', \
                   filePath.encode(globalref.localTextEncoding)
-            self.fileInfoFormat = origFileInfoFormat
             raise
         except UnicodeError:
             print 'Error - bad Unicode in file', \
                   filePath.encode(globalref.localTextEncoding)
             f.close()
-            self.fileInfoFormat = origFileInfoFormat
             raise
         except xml.sax.SAXException:
-            self.storedFileRef = f
-            self.storedFileRef.seek(0)
-            self.fileInfoFormat = origFileInfoFormat
+            f.close()
             raise ReadFileError(_('Could not open as treeline file'))
         f.close()
         self.root = handler.rootItem
         self.fileName = filePath
         self.treeFormats = TreeFormats(handler.formats)
-        self.xlstLink = handler.xlstLink
-        self.xslCssLink = handler.xslCssLink
-        self.spaceBetween = handler.spaceBetween
-        self.lineBreaks = handler.lineBreaks
-        self.formHtml = handler.formHtml
-        self.childFieldSep = handler.childFieldSep
-        self.spellChkLang = handler.spellChkLang
-        self.tlVersion = handler.tlVersion
-        self.selection = treeselection.TreeSelection([self.root])
-        self.undoStore = undo.UndoRedoStore()
-        self.redoStore =  undo.UndoRedoStore()
-        self.modified = False
-        self.sortFields = ['']
         self.fileInfoFormat.replaceListFormat()
-        self.fileInfoFormat.translateFields()
-        self.fileInfoFormat.updateFileInfo()
         self.treeFormats.updateAutoChoices()
         self.treeFormats.updateUniqueID()
         self.treeFormats.updateDerivedTypes()
@@ -256,7 +238,6 @@ class TreeDoc(object):
                 f.close()
             return
         f.close()
-        origFormats = self.treeFormats
         bufList = [(text.count('\t', 0, len(text) - len(text.lstrip())),
                     text.strip()) for text in textList if text.strip()]
         if bufList:
@@ -269,10 +250,7 @@ class TreeDoc(object):
                 if newRoot.loadTabbedChildren(bufList):
                     self.root = newRoot
                     self.fileName = filePath
-                    self.resetParam()
                     return
-        self.treeFormats = origFormats
-        self.modified = False
         raise ReadFileError(_('Error in tabbed list'))
 
     def readTable(self, fileRef, errors='strict'):
@@ -290,7 +268,6 @@ class TreeDoc(object):
                 f.close()
             return
         f.close()
-        origFormats = self.treeFormats
         self.treeFormats = TreeFormats({}, True)  # set defaults ROOT & DEFAULT
         newRoot = TreeItem(None, TreeFormats.rootFormatDefault)
         defaultFormat = self.treeFormats[TreeFormats.formatDefault]
@@ -307,13 +284,10 @@ class TreeDoc(object):
                     newItem.data[self.treeFormats[TreeFormats.formatDefault].
                             fieldList[num].name] = lineList[num].strip()
             except IndexError:
-                self.treeFormats = origFormats
-                self.modified = False
                 print 'Too few headings to read data as a table'
                 raise ReadFileError(_('Too few headings to read data as table'))
         self.root = newRoot
         self.fileName = filePath
-        self.resetParam()
 
     def readLines(self, fileRef, errors='strict'):
         """Import plain text, node per line"""
@@ -345,7 +319,6 @@ class TreeDoc(object):
                 newItem.data[TreeFormats.textFieldName] = line
         self.root = newRoot
         self.fileName = filePath
-        self.resetParam()
 
     def readPara(self, fileRef, errors='strict'):
         """Import plain text, blank line delimitted"""
@@ -382,7 +355,6 @@ class TreeDoc(object):
                 newItem.data[TreeFormats.textFieldName] = line
         self.root = newRoot
         self.fileName = filePath
-        self.resetParam()
 
     def readTreepad(self, fileRef, errors='strict'):
         """Read Treepad text-node file"""
@@ -399,7 +371,6 @@ class TreeDoc(object):
             else:
                 f.close()
             return
-        origFormats = self.treeFormats
         self.treeFormats = TreeFormats()
         format = nodeformat.NodeFormat(TreeFormats.formatDefault)
         titleFieldName = _('Title', 'title field name')
@@ -422,8 +393,6 @@ class TreeDoc(object):
                     level = int(lines[1])
                     lines = lines[2:]
                 except (ValueError, IndexError):
-                    self.treeFormats = origFormats
-                    self.modified = False
                     print 'Error - bad file format in %s' % \
                           filePath.encode(globalref.localTextEncoding)
                     raise ReadFileError(_('Bad file format in %s') % filePath)
@@ -442,7 +411,6 @@ class TreeDoc(object):
             parentList.append(item)
         self.root = itemList[0]
         self.fileName = filePath
-        self.resetParam()
 
     def createBookmarkFormat(self):
         """Return a set of formats for bookmark imports"""
@@ -502,7 +470,6 @@ class TreeDoc(object):
                       TreeDoc.bookmarkRootTitle
         self.fileName = filePath
         self.treeFormats = formats
-        self.resetParam()
 
     def readMozilla(self, fileRef, errors='strict'):
         """Read Mozilla HTML format bookmarks"""
@@ -535,7 +502,6 @@ class TreeDoc(object):
                       TreeDoc.bookmarkRootTitle
         self.fileName = filePath
         self.treeFormats = formats
-        self.resetParam()
 
     def readXml(self, fileRef):
         """Read a generic (non-TreeLine) XML file"""
@@ -567,11 +533,9 @@ class TreeDoc(object):
         for format in self.treeFormats.values():
             format.fixImportedFormat(treexmlparse.GenericXmlHandler.
                                                   textFieldName)
-        self.resetParam()
 
     def readOdf(self, fileRef):
         """Read an Open Document Format (ODF) file"""
-        origFormats = self.treeFormats
         self.treeFormats = TreeFormats(None, True)
         rootItem = TreeItem(None, TreeFormats.rootFormatDefault,
                             TreeDoc.rootTitleDefault)
@@ -591,25 +555,21 @@ class TreeDoc(object):
             xml.sax.parseString(text, handler)
         except (zipfile.BadZipfile, KeyError):
             f.close()
-            self.treeFormats = origFormats
             raise ReadFileError(_('Could not unzip ODF file'))
         except UnicodeError:
             f.close()
-            self.treeFormats = origFormats
             raise ReadFileError(_('Problem with Unicode characters in file'))
         except xml.sax.SAXException:
             f.close()
-            self.treeFormats = origFormats
             raise ReadFileError(_('Could not open corrupt ODF file'))
         f.close()
         self.root = rootItem
         self.fileName = filePath
-        self.resetParam()
 
     def readXmlString(self, string):
         """Read xml string and return top item or None"""
         try:
-            handler = treexmlparse.TreeSaxHandler()
+            handler = treexmlparse.TreeSaxHandler(self)
             xml.sax.parseString(string.encode('utf-8'), handler)
         except xml.sax.SAXException:
             return None
@@ -618,7 +578,7 @@ class TreeDoc(object):
     def readXmlStringAndFormat(self, string):
         """Read xml string and return tuple of top item and new formats"""
         try:
-            handler = treexmlparse.TreeSaxHandler()
+            handler = treexmlparse.TreeSaxHandler(self)
             xml.sax.parseString(string.encode('utf-8'), handler)
         except xml.sax.SAXException:
             return (None, [])
@@ -727,7 +687,7 @@ class TreeDoc(object):
             raise
         f.close()
 
-    def exportDir(self, dirName, nodeList, addHeader=False):
+    def exportDirTable(self, dirName, nodeList, addHeader=False):
         """Write tree to nested directory struct with html tables"""
         oldDir = os.getcwd()
         os.chdir(dirName.encode(sys.getfilesystemencoding()))
@@ -746,8 +706,37 @@ class TreeDoc(object):
         else:
             item = nodeList[0]
         linkDict = {}
-        item.createDirLinkDict(linkDict, os.getcwd())
-        item.exportDir(linkDict, None, header, footer)
+        item.createDirTableLinkDict(linkDict, os.getcwd())
+        item.exportDirTable(linkDict, None, header, footer)
+        self.treeFormats.removeQuiet(TreeDoc.copyFormat)
+        os.chdir(oldDir)
+
+    def exportDirPage(self, dirName, nodeList):
+        """Write tree to nested direct struct with html page for each node"""
+        oldDir = os.getcwd()
+        os.chdir(dirName.encode(sys.getfilesystemencoding()))
+        cssLines = ['#sidebar {', 'width: 16em;', 'float: left;',
+                    'clear: left;', 'border-right: 1px solid black;',
+                    'margin-right: 1em;', '}']
+        try:
+            f = codecs.open('default.css', 'w', 'utf-8')
+            f.writelines([(line + '\n').encode('utf-8') for line in cssLines])
+        except (IOError, UnicodeError):
+            print 'Error - could not write file to default.css'
+            raise IOError(_('Error - cannot write file to %s') % 'default.css')
+        f.close()
+        if len(nodeList) > 1:
+            self.treeFormats.addIfMissing(TreeDoc.copyFormat)
+            item = TreeItem(None, TreeDoc.copyFormat.name)
+            item.data[TreeFormats.fieldDefault] = TreeDoc.rootTitleDefault
+            for child in nodeList:
+                item.childList.append(child)
+                child.parent = item
+        else:
+            item = nodeList[0]
+        linkDict = {}
+        item.createDirPageLinkDict(linkDict, os.getcwd())
+        item.exportDirPage(linkDict)
         self.treeFormats.removeQuiet(TreeDoc.copyFormat)
         os.chdir(oldDir)
 
