@@ -5,7 +5,7 @@
  treeline.py, the main program file
 
  TreeLine, an information storage program
- Copyright (C) 2006, Douglas W. Bell
+ Copyright (C) 2011, Douglas W. Bell
 
  This is free software; you can redistribute it and/or modify it under the
  terms of the GNU General Public License, either Version 2 or any later
@@ -15,7 +15,7 @@
 """
 
 __progname__ = 'TreeLine'
-__version__ = '1.2.4'
+__version__ = '1.4.0'
 __author__ = 'Doug Bell'
 
 helpFilePath = None    # modified by install script if required
@@ -129,6 +129,7 @@ def main():
 
     import treedoc
     from cmdline import CmdLine
+    import treecontrol
     import treemainwin
 
     if not treedoc.testXmlParser():
@@ -145,20 +146,12 @@ def main():
         sys.exit(2)
     args = args[1:]
 
+    treeControl = treecontrol.TreeControl(userStyle)
+
     if opts:
         CmdLine(opts, args)
     else:
-        if not userStyle:
-            if sys.platform.startswith('dar'):
-                QtGui.QApplication.setStyle('macintosh')
-            elif not sys.platform.startswith('win'):
-                QtGui.QApplication.setStyle('plastique')
-        win = treemainwin.TreeMainWin()
-        if args:
-            win.openFile(unicode(args[0], globalref.localTextEncoding))
-        else:
-            win.autoOpen()
-        win.show()
+        treeControl.firstWindow(args)
         signal.signal(signal.SIGINT, signal.SIG_IGN)
         app.exec_()
 
